@@ -1,24 +1,59 @@
 import sys
 from threading import Thread
+from time import sleep
+from typing import Iterable
 
-from PySide6.QtCore import QObject, QCoreApplication, QUrl, qInstallMessageHandler, Slot, Signal
+from PySide6.QtCore import (
+    QObject,
+    QCoreApplication,
+    QUrl,
+    qInstallMessageHandler,
+    Slot,
+    Signal,
+)
 from PySide6.QtGui import Qt, QIcon
 from PySide6.QtQml import QQmlApplicationEngine
 from PySide6.QtWidgets import QApplication
 
+import dvpn.res  # noqa
 from dvpn.config.constants import PublicVars, CLI_RESOLVE
-from dvpn.modules.tools import connect
 from dvpn.logger import qt_message_handler
-import res  # noqa
+from dvpn.modules.tools import connect
 
 
 class Bridge(QObject):
-    connectedVPNs = set()
+    periodic_thread = None
+    connectedVPNs: Iterable[str] = set()
     changingVPNs = set()
 
     connectStatusChange = Signal(str, bool, bool, name="connectStatusChange")
     disconnectChange = Signal(str, bool, bool, name="disconnectChange")
 
+    def periodic_check(self):
+        if self.periodic_thread is not None:
+            return False
+        t = Thread(target=self._periodic_check, name="Periodic status Check", daemon=True)
+        self.periodic_thread = t
+        t.start()
+        return True
+
+    def _periodic_check(self):
+        while True:
+            for vpn_name in set(self.connectedVPNs):
+                # Wait for previous operation to finish
+                while vpn_name in self.changingVPNs:
+                    sleep(0.1)
+
+                creds = PublicVars().credentials[vpn_name]
+                cli_type = CLI_RESOLVE[creds["selectedVpn"]]
+                cli_instance = cli_type(creds["cliPath"])
+                state = cli_instance.get_state(vpn_name).lower()
+
+                if "disconnected" in state:
+                    self.disconnect_notify(vpn_name, cli_instance, False)
+
+            sleep(0.5)
+
     @Slot(str)
     def log(self, text):
         print(text)
@@ -56,8 +91,11 @@ class Bridge(QObject):
 
         vpn_conf = PublicVars()[vpn_name]
         cli = CLI_RESOLVE[vpn_conf["selectedVpn"]](vpn_conf["cliPath"])
-        t = Thread(target=lambda: connect(cli, vpn_conf["VPN Name"], self),
-                   name=f"Connecting {vpn_conf['VPN Name']}", daemon=True)
+        t = Thread(
+            target=lambda: connect(cli, vpn_conf["VPN Name"], self),
+            name=f"Connecting {vpn_conf['VPN Name']}",
+            daemon=True,
+        )
         t.start()
 
     @Slot(str, "QVariantMap")
@@ -76,35 +114,45 @@ class Bridge(QObject):
     def connected_notify(self, vpn_name: str):
         self.connectedVPNs.add(vpn_name)
 
+
     @Slot(str)
     def disconnect(self, vpn_name: str):
         # noinspection PyUnresolvedReferences
         self.disconnectChange.emit(vpn_name, True, True)
         vpn_conf = PublicVars()[vpn_name]
         cli = CLI_RESOLVE[vpn_conf["selectedVpn"]](vpn_conf["cliPath"])
-        t = Thread(target=lambda: self.disconnect_notify(vpn_name, cli),
-                   name=f"Disconnecting {vpn_name}", daemon=True)
+        t = Thread(
+            target=lambda: self.disconnect_notify(vpn_name, cli),
+            name=f"Disconnecting {vpn_name}",
+            daemon=True,
+        )
         t.start()
 
     @Slot()
     def reset(self):
         for host in set(self.connectedVPNs):
+            self.changingVPNs.add(host)
             creds = PublicVars().credentials[host]
             cli_type = CLI_RESOLVE[creds["selectedVpn"]]
             cli = cli_type(str(creds["cliPath"]))
-            t = Thread(target=lambda: self.disconnect_notify(host, cli),
-                       name=f"Disconnecting {host}", daemon=True)
+            t = Thread(
+                target=lambda: self.disconnect_notify(host, cli),
+                name=f"Disconnecting {host}",
+                daemon=True,
+            )
             t.start()
 
-    def disconnect_notify(self, vpn_name, cli):
+    def disconnect_notify(self, vpn_name, cli, reset=True):
         self.disconnectChange.emit(vpn_name, False, True)
-        cli.reset(host=vpn_name)
+        if reset:
+            cli.reset(host=vpn_name)
         # noinspection PyUnresolvedReferences
         self.disconnectChange.emit(vpn_name, False, False)
-        self.connectedVPNs.remove(vpn_name)
+        self.connectedVPNs.discard(vpn_name)
+        self.changingVPNs.discard(vpn_name)
 
 
-def open_gui():
+def main():
     qInstallMessageHandler(qt_message_handler)
     QCoreApplication.setAttribute(Qt.AA_EnableHighDpiScaling)
     QCoreApplication.setAttribute(Qt.AA_UseHighDpiPixmaps)
@@ -115,6 +163,7 @@ def open_gui():
     app.setApplicationName("DieVpn")
     engine = QQmlApplicationEngine()
     bridge = Bridge()
+    bridge.periodic_check()
     context = engine.rootContext()
     context.setContextProperty("con", bridge)
     engine.load(QUrl("qrc:/qml/main.qml"))
@@ -124,4 +173,4 @@ def open_gui():
 
 
 if __name__ == "__main__":
-    open_gui()
+    main()
